#ifndef JPKINTERFACE_H_INCLUDED
#define JPKINTERFACE_H_INCLUDED
//------------------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif


#include <tchar.h>
#include <windows.h>

typedef void (__stdcall *PROGRESS_PROC)(void* SessionID,
                                        __int64 FilesCount,
                                        __int64  CurrentFile,
                                        __int64  FileSize,
                                        __int64  CurrentPosition,
                                        int* UserBreak,
                                        int Err,
                                        Wchar* ErrMsg);

typedef void (__stdcall *FINISH_PROC)(void*  SessionID,
                                      Wchar* ReferenceNumber,
                                      int Status,
                                      Wchar* StatusText,
                                      Wchar* UPO,
                                      int UserBreak,
                                      int Err,
                                      Wchar* ErrMsg);

typedef void __stdcall (*TSetLic)(WCHAR* licText);
typedef void __stdcall (*TSetLicA)(char* licText);

typedef void __stdcall (*TSetPublicKeyFile)(WCHAR* KeyFileName,
                                            WCHAR* KeyPass);

typedef void __stdcall (*TSetPublicKeyFileA)(WCHAR* KeyFileName,
                                             WCHAR* KeyPass);

typedef void __stdcall (*TSetCertFile)(WCHAR* CertFileNameOdIndex,
                                       WCHAR* CertPass);

typedef int __stdcall (*TGetCertCurrentIndex)(void);
typedef int __stdcall (*TShowCertList)(unsigned int ParentHandle);
typedef void __stdcall (*TSetPublicKey)(void* Key, int KeySize, WCHAR* KeyPass);
typedef void __stdcall (*TSetCert)(void* Cert, int CertSize, WCHAR* CertPass);
typedef int __stdcall (*TSetProgressCaption)(void* SessionID, WCHAR* Caption);
typedef void __stdcall (*TSetParentwindowHandle)(unsigned int Handle);
typedef void __stdcall (*TInternetSettingsForm)(unsigned int ParentHandle);
typedef int __stdcall (*TGetCertyficateCount)(void);
typedef int __stdcall (*TGetCertyficate)(int Index, WCHAR** cert);

typedef int __stdcall (*TAddXMLJPK)(char* XML,
                                    unsigned char DocType,
                                    WCHAR* DefFileName);

typedef int __stdcall (*TAddXMLJPKFile)(wchar* XMLFileName,
                                        unsigned char DocType);

typedef LPVOID __stdcall (*TSignSendJPK)(int TestMode,
                                         PROGRESS_PROC Progress,
                                         FINISH_PROC Finish,
                                         int SyncFinish);

typedef int __stdcall (*TSignSendJPKAndWait)(int TestMode,
                                             PROGRESS_PROC Progress,
                                             Wchar**ReferenceNumbers);

typedef void __stdcall (*TSignSendBreak)(void* SessionID);

typedef void __stdcall (*TWaitForEnd)(void* SessionID,
                                      WCHAR** ReferenceNumbers);

typedef Wchar* __stdcall (*TGetReferenceNumbers)(WCHAR* SessionID);

typedef int __stdcall (*TGetUPO)(int TestMode,
                                 WCHAR* ReferenceNumber,
                                 WCHAR** UPO,
                                 int* Status,
                                 WCHAR** statusText);

typedef int __stdcall (*TBreakGetUPO)(WCHAR* ReferenceNumber);
typedef void __stdcall (*TJPKGetLastError)(WCHAR** ErrText);

typedef int __stdcall (*TFileNameValidate)(const WCHAR* FileName,
                                           WCHAR** errText);

typedef int __stdcall (*TXMLValidate)(const char* XMLData,
                                      WCHAR** errText);

typedef int __stdcall (*TXMLFileValidate)(const WCHAR* FileName,
                                          WCHAR** errText);

typedef void __stdcall (*TXMLValidateBreak)(void);
typedef int __stdcall (*TPrintUPO)(WCHAR* UPO, unsigned int ParentHandle);
typedef void __stdcall (*TFinalizeDLL)(void);

TSetLic SetLic = NULL;
TSetLicA SetLicA = NULL;
TSetPublicKeyFile SetPublicKeyFile = NULL;
TSetPublicKeyFileA SetPublicKeyFileA = NULL;
TSetCertFile SetCertFile = NULL;
TGetCertCurrentIndex GetCertCurrentIndex = NULL;
TShowCertList ShowCertList = NULL;
TSetPublicKey SetPublicKey = NULL;
TSetCert SetCert = NULL;
TSetProgressCaption SetProgressCaption = NULL;
TSetParentwindowHandle SetParentwindowHandle = NULL;
TInternetSettingsForm InternetSettingsForm = NULL;
TGetCertyficateCount GetCertyficateCount = NULL;
TGetCertyficate GetCertyficate = NULL;
TAddXMLJPK AddXMLJPK = NULL;
TAddXMLJPKFile AddXMLJPKFile = NULL;
TSignSendJPK SignSendJPK = NULL;
TSignSendJPKAndWait SignSendJPKAndWait = NULL;
TSignSendBreak SignSendBreak = NULL;
TWaitForEnd WaitForEnd = NULL;
TGetReferenceNumbers GetReferenceNumbers = NULL;
TGetUPO GetUPO = NULL;
TBreakGetUPO BreakGetUPO = NULL;
TJPKGetLastError JPKGetLastError = NULL;
TFileNameValidate FileNameValidate = NULL;
TXMLValidate XMLValidate = NULL;
TXMLFileValidate XMLFileValidate = NULL;
TXMLValidateBreak XMLValidateBreak = NULL;
TPrintUPO PrintUPO = NULL;
TFinalizeDLL FinalizeDLL = NULL;

const char* JPKLibDLLName = "JPKLibUI.dll";
const char* cRegJPK = "Software\\IPSPI\\JotPeK";
const char* cRegJPKPathKeyName = "Path";
const char* JPKDLLFullPath = "";
HINSTANCE vJPKLib = NULL;

static void FreeJPKDLL(void)
{
    if (vJPKLib)
        FreeLibrary(vJPKLib);
    vJPKLib = NULL;
}

static void* GetPA(char* procName)
{
    void* p = GetProcAddress(vJPKLib, procName);
    if (!p)
        FreeJPKDLL();
    return p;
}

static void InitJPKDLL(void)
{
    char vDLLPath[PATH_MAX];
    if (!strcmp(JPKDLLFullPath, ""))
    {
        GetModuleFileName(NULL, vDLLPath, PATH_MAX);
        int pos = strrchr(vDLLPath, '\\') - vDLLPath;
        vDLLPath[pos] = 0;
    }
    else
    {
        DWORD size = PATH_MAX;
        RegGetValue(HKEY_LOCAL_MACHINE,
                    cRegJPK,
                    cRegJPKPathKeyName,
                    RRF_RT_ANY, NULL,
                    (PVOID)&vDLLPath,
                    &size);
    }
    char vDLLFullPath[PATH_MAX] = {0};
    strcat(vDLLFullPath, vDLLPath);
    int pos = strlen(vDLLFullPath);
    if (vDLLFullPath[pos-1] != '\\')
    {
        vDLLFullPath[pos-1] = '\\';
        vDLLFullPath[pos] = 0;
    }
    strcat(vDLLFullPath, JPKLibDLLName);
    vJPKLib = LoadLibrary(vDLLFullPath);
    if (vJPKLib)
    {
        SetLic = GetPA("SetLic");
        SetPublicKeyFile = GetPA("SetPublicKeyFile");
        SetCertFile = GetPA("SetCertFile");
        GetCertCurrentIndex = GetPA("GetCertCurrentIndex");
        ShowCertList = GetPA("ShowCertList");
        SetPublicKey = GetPA("SetPublicKey");
        SetCert = GetPA("SetCert");
        SetProgressCaption = GetPA("SetProgressCaption");
        SetParentwindowHandle = GetPA("SetParentwindowHandle");
        InternetSettingsForm = GetPA("InternetSettingsForm");
        GetCertyficateCount = GetPA("GetCertyficateCount");
        GetCertyficate = GetPA("GetCertyficate");
        AddXMLJPK = GetPA("AddXMLJPK");
        AddXMLJPKFile = GetPA("AddXMLJPKFile");
/*
        AddXMLJPKex = GetPA("AddXMLJPKex");
        AddXMLJPKFileEx = GetPA("AddXMLJPKFileEx");
*/
        SignSendJPK = GetPA("SignSendJPK");
        SignSendJPKAndWait = GetPA("SignSendJPKAndWait");
        SignSendBreak = GetPA("SignSendBreak");

        WaitForEnd = GetPA("WaitForEnd");
        GetReferenceNumbers = GetPA("GetReferenceNumbers");
        GetUPO = GetPA("GetUPO");
        BreakGetUPO = GetPA("BreakGetUPO");
        JPKGetLastError = GetPA("JPKGetLastError");

        FileNameValidate = GetPA("FileNameValidate");
        XMLValidate = GetPA("XMLValidate");
        XMLFileValidate = GetPA("XMLFileValidate");
        XMLValidateBreak = GetPA("XMLValidateBreak");
        PrintUPO = GetPA("PrintUPO");
        FinalizeDLL = GetPA("FinalizeDLL");
    }
}



#ifdef __cplusplus
}
#endif
///-----------------------------------------------------------------------------
#endif // JPKINTERFACE_H_INCLUDED
